? my ($app, $window, @classes) = @_;
? my $active = grep {$_ eq "active"} @classes;
<div class="window <?= join " ", @classes ?>" id="<?= $window->id ?>">
  <table cellspacing="0" cellpadding="0">
    <tr class="topic">
      <td id="<?= $window->id ?>_topic">
        <?= $window->topic->{string} ?>
      </td>
    </tr>
    <tr>
      <td valign="top">
        <ul class="messages" id="<?= $window->id ?>_messages">
        </ul>
      </td>
    </tr>
    <tr class="vert_dragger">
      <td id="<?= $window->id ?>_dragger"></td>
    </tr>
    <tr class="input">
      <td id="<?= $window->id ?>_input">
        <form id="<?= $window->id ?>_form" autocomplete="off">
          <input type="hidden" name="source" value="<?= $window->id ?>" />
          <button type="submit" style="display:none" id="<?= $window->id ?>_submit">Send</button>
          <textarea name="msg" id="<?= $window->id ?>_msg" rows="1"></textarea>
        </form>
      </td>
    </tr>
  </table>
</div>
